#include "StdAfx.h"
#include ".\ExpCondition.h"
#include "Globals.h"

//using namespace System;

extern int g_separatorComma;

IMPLEMENT_SERIAL( ExpCondition, CObject, 1 )


ExpCondition::ExpCondition(CString name)
: m_columnAV(0)
, m_result(0)
, m_size(0)
, m_aVsSize(0)
, m_firstLine(0)
, m_lastLine(0)
, m_centTendency(0)
, m_specialAv(FALSE)
, m_lineToFileEnd(FALSE)
, m_popEst(TRUE)
{
	m_name = name;
	
	addSubCondition( 0, 0, 1.0 );
	m_computedAv = new ComputedAv();
}

ExpCondition::ExpCondition( ExpCondition *copyExpCondition )
{
	m_size = copyExpCondition->returnSize( );
	m_aVsSize = copyExpCondition->m_aVsSize;
	subCondType.Copy( copyExpCondition->subCondType );
	m_columnAV = copyExpCondition->m_columnAV;
	m_result = copyExpCondition->m_result;
	m_name = copyExpCondition->m_name;
	m_firstLine = copyExpCondition->m_firstLine;
	m_lastLine = copyExpCondition->m_lastLine;
	m_centTendency = copyExpCondition->m_centTendency;

	m_computedAv = new ComputedAv();

	m_specialAv = copyExpCondition->retSpecialAv();
	m_lineToFileEnd = copyExpCondition->returnLineToFileEnd();
	m_popEst = copyExpCondition->returnPopEst();
	m_computedAv = new ComputedAv();

	for( int i=0; i < copyExpCondition->m_computedAv->columns.GetSize(); i++ )
	{
		m_computedAv->columns.Add( copyExpCondition->m_computedAv->columns.GetAt( i ) );
	}
	
	m_computedAv->operation = copyExpCondition->m_computedAv->operation;
	

	for (int i=0; i < copyExpCondition->returnSize( ); i++)
	{
		if (copyExpCondition->at( i )->orType)
		{
			OrSubCondition *theOrSubCondition = new OrSubCondition( ((OrSubCondition *)(copyExpCondition->at( i ))) );
			subConditions.Add( theOrSubCondition );
		}
		else
		{
			subConditions.Add( new SubCondition( copyExpCondition->at( i ) ) );
		}
		
	}
}

ExpCondition::ExpCondition( ExpCondition *copyExpCondition, int foo )
{
	m_size = 0;
	m_aVsSize = 0;
	m_columnAV = copyExpCondition->m_columnAV;;
	m_name = "";
	m_firstLine = copyExpCondition->m_firstLine;
	m_lastLine = copyExpCondition->m_lastLine;
	m_centTendency = copyExpCondition->m_centTendency;

	m_specialAv = copyExpCondition->retSpecialAv();
	m_lineToFileEnd = copyExpCondition->returnLineToFileEnd();
	m_popEst = copyExpCondition->returnPopEst();
	m_computedAv = new ComputedAv();

	for( int i=0; i < copyExpCondition->m_computedAv->columns.GetSize(); i++ )
	{
		m_computedAv->columns.Add( copyExpCondition->m_computedAv->columns.GetAt( i ) );
	}
	
	m_computedAv->operation = copyExpCondition->m_computedAv->operation;

	addSubCondition( 0, 0, 1.0 );
}


ExpCondition::~ExpCondition(void)
{
	for (int i=0; i < returnSize( ); i++)
	{
		delete at( i );
	}
}

BOOL ExpCondition::checkLine(WTextFile *theTextFile, int lineNumber)
{
	// If there is no limiting rule, the line of the protocol files is included into the further calculations.;
	// For this reason: no catch of returnSize() == 0, the for loop will be not run in this case,
	// the value of Av will be included in result.
	
	for ( int i=0; i < returnSize(); i++ )
	{
		// If a subcondition is not met, the row is excluded from calculations. 
		if ( ! at( i )->check( theTextFile, lineNumber ) )
			return FALSE;
	}

	double result;

	if( ! m_specialAv )
	{
		// If m_columnAV is valid, e.g. not the case with # incidences
		if( m_columnAV > -1 )
		{
			// Check for valid value
			if( wcscmp( theTextFile->at( lineNumber, m_columnAV )->c_str( ), L"" ) != 0 )
				result = _wtof( theTextFile->at( lineNumber, m_columnAV )->c_str( ) );	
		}
		else result = 0.0f; // Dummy-value for # incidences
	}
	else
	{
		switch( m_computedAv->operation )
		{
			case 0: // sum
				result = 0.0;

				for( int i=0; i < m_computedAv->columns.GetSize(); i++ )
				{
					result += _wtof( theTextFile->at( lineNumber, (m_computedAv->columns.GetAt( i )) - 1 )->c_str( ) );
				}
				break;

			case 1: // difference

				// Take first element and subtract all others.
				result = _wtof( theTextFile->at( lineNumber, (m_computedAv->columns.GetAt( 0 )) - 1 )->c_str( ) );

				for( int i=1; i < m_computedAv->columns.GetSize(); i++ )
				{
					result -= _wtof( theTextFile->at( lineNumber, (m_computedAv->columns.GetAt( i )) - 1 )->c_str( ) );
				}
				break;

			case 2: // mean
				result = 0.0;

				for( int i=0; i < m_computedAv->columns.GetSize(); i++ )
				{
					result += _wtof( theTextFile->at( lineNumber, (m_computedAv->columns.GetAt( i )) - 1 )->c_str( ) );
				}

				result /= (double)(m_computedAv->columns.GetSize());

				break;

			case 3: // median
				// First, extract all values ...
				CArray<double> numbers;

				for( int i=0; i < m_computedAv->columns.GetSize(); i++ )
				{
					numbers.Add( _wtof( theTextFile->at( lineNumber, (m_computedAv->columns.GetAt( i )) - 1 )->c_str( ) ) );
				}


				// Sort
				double buffer;
				BOOL repeat = true;
				
				while( repeat )
				{
					repeat = false;
					for( int i=0; i <  numbers.GetSize() - 1; i++ )
					{
						if( numbers.GetAt( i ) > numbers.GetAt( i+1 ) )
						{
							repeat = true;
							buffer = numbers.GetAt( i );
							numbers.SetAt( i, numbers.GetAt( i+1 ) );
							numbers.SetAt( i+1, buffer );
						}
					}
				}


				// Even or uneven number of elements?
				int n = numbers.GetSize();
				if( n % 2 != 0 ) // uneven
				{
					result = numbers.GetAt( ((n + 1) / 2) - 1 ); // -1 due to basis 0 of array
				}
				else // even
				{
					result = 0.5 * ( numbers.GetAt( (n/2) - 1 ) + numbers.GetAt( (n/2) + 1 - 1 ) );
				}

				break;
			} // switch statement
	}

	m_relevantAVs.Add( result );

	m_aVsSize++;
	return true;
}

double ExpCondition::computeMean( int *errorCode )
{
	if( returnAVsSize( ) < 1 )
	{
		*errorCode = 1;
		return 0.0f;
	}
	else
		*errorCode = 0;
	
	double sum = 0;
	for (int i=0; i < returnAVsSize( ); i++)
	{
		sum += AVsAt(i);
	}

	m_result = sum / (double)(returnAVsSize( ));

	return m_result;
}

void ExpCondition::computeRawValues( int *errorCode, CArray<double> &rawValues )
{
	if( returnAVsSize( ) < 1 )
	{
		*errorCode = 1;
	}
	else
		*errorCode = 0;

	rawValues.RemoveAll( );

	for (int i=0; i < returnAVsSize( ); i++)
	{
		rawValues.Add( AVsAt(i) );
	}
}

int ExpCondition::computeNumberOfIncidences( int *errorCode )
{
	*errorCode = 0;
	
	m_result = (double)(m_relevantAVs.GetSize());
	return m_result;
}

double ExpCondition::computeMedian( int *errorCode )
{
	if( returnAVsSize( ) < 1 )
	{
		*errorCode = 1;
		return 0.0f;
	}
	else
		*errorCode = 0;
	
	// Sort
	double buffer;
	BOOL repeat = true;
	
	while( repeat )
	{
		repeat = false;
		for( int i=0; i <  returnAVsSize( ) - 1; i++ )
		{
			if( AVsAt( i ) > AVsAt( i+1 ) )
			{
				repeat = true;
				buffer = AVsAt( i );
				AVsAt( i ) = AVsAt( i+1 );
				AVsAt( i+1 ) = buffer;
			}
		}
	}


	int n = returnAVsSize( );
	if( n == 0 )
	{
		double foo = 1e-400;
		return (double)((int)1 / foo);
	}
	// Even or uneven number of elements?
	if( n % 2 != 0 ) // uneven
	{
		m_result = AVsAt( (( n + 1 ) / 2) - 1 ); // -1 due to basis 0 of arraz
	}
	else // even
	{
		m_result = 0.5 * ( AVsAt( (n/2)-1 ) + AVsAt( (n/2)+1-1 ) );
	}

	return m_result;
}

double ExpCondition::computeSum( int *errorCode )
{
	if( returnAVsSize( ) < 1 )
	{
		*errorCode = 1;
		return 0.0f;
	}
	else
		*errorCode = 0;
	
	double sum = 0;
	for (int i=0; i < returnAVsSize( ); i++)
	{
		sum += AVsAt(i);
	}

	m_result = sum;

	return m_result;
}

double ExpCondition::computeVariance( BOOL populationEstimation, int *errorCode )
{
	if( returnAVsSize( ) < 1 )
	{
		*errorCode = 1;
		return 0.0f;
	}
	else
	{
		double mean = computeMean( errorCode );
		double sum = 0;

		for (int i=0; i < returnAVsSize( ); i++)
		{
			sum += ( (AVsAt(i) - mean) * (AVsAt(i) - mean) );
		}

		if( populationEstimation )
		{
			if( returnAVsSize( ) < 2 )
			{
				*errorCode = 1;
				return 0.0f;
			}
			
			m_result = sum / (double)(returnAVsSize( ) - 1);
		}
		else
			m_result = sum / (double)(returnAVsSize( ));

		return m_result;
	}
}

double ExpCondition::computeStandardDeviation( BOOL populationEstimation,  int *errorCode )
{
	double variance = computeVariance( populationEstimation, errorCode );

	m_result = sqrt( variance );

	return m_result;
}

double ExpCondition::returnResult( int *errorCode )
{
	switch( m_centTendency )
	{
		case CENT_TEND_MEAN:
			computeMean( errorCode );
		break;
		case CENT_TEND_MEDIAN:
			computeMedian( errorCode );
		break;
		case CENT_TEND_INCIDENCES:
			computeNumberOfIncidences( errorCode );
		break;
		case CENT_TEND_SUM:
			computeSum( errorCode );
		break;
		case CENT_TEND_VARIANCE:
			computeVariance( m_popEst, errorCode );
		break;
		case CENT_TEND_STANDDEV:
			computeStandardDeviation( m_popEst, errorCode );
		break;
	}
	

	return m_result;
}

SubCondition* ExpCondition::addSubCondition(int column, int comparator, double referenceValue)
{
	SubCondition *theSubCondition = new SubCondition( column, comparator, referenceValue );
	subConditions.Add( theSubCondition );
	subCondType.Add( 0 );
	m_size++;

	return theSubCondition;
}

SubCondition* ExpCondition::addSubCondition(int column, int comparator, CString referenceValueText)
{
	SubCondition *theSubCondition = new SubCondition( column, comparator, referenceValueText );
	subConditions.Add( theSubCondition );
	subCondType.Add( 0 );
	m_size++;

	return theSubCondition;
}

SubCondition* ExpCondition::insertSubCondition(int where, int column, int comparator, double referenceValue)
{
	SubCondition *theSubCondition = new SubCondition( column, comparator, referenceValue );
	subConditions.InsertAt( where, theSubCondition );
	subCondType.InsertAt( where, 0 );
	m_size++;

	return theSubCondition;
}

SubCondition* ExpCondition::insertSubCondition(int where, int column, int comparator, CString referenceValueText)
{
	SubCondition *theSubCondition = new SubCondition( column, comparator, referenceValueText );
	subConditions.InsertAt( where, theSubCondition );
	subCondType.InsertAt( where, 0 );
	m_size++;

	return theSubCondition;
}


OrSubCondition* ExpCondition::addOrSubCondition()
{
	OrSubCondition *theOrSubCondition = new OrSubCondition( 0, 0, 1);
	subConditions.Add( theOrSubCondition );
	subCondType.Add( 1 );
	m_size++;

	return theOrSubCondition;
}

OrSubCondition* ExpCondition::insertOrSubCondition( int where )
{
	OrSubCondition *theOrSubCondition = new OrSubCondition( 0, 0, 1);

	if( where < subConditions.GetSize() )
	{
		subConditions.InsertAt( where, theOrSubCondition );
		subCondType.InsertAt( where, 1 );
	}
	else
	{
		subConditions.Add( theOrSubCondition );
		subCondType.Add( 1 );
	}

	m_size++;

	return theOrSubCondition;
}


void ExpCondition::deleteLastSubCondition()
{
	delete at( returnSize( ) - 1 );
	subConditions.RemoveAt( returnSize( ) - 1 );
	subCondType.RemoveAt( returnSize( ) - 1 );
	m_size--;
}

void ExpCondition::deleteSubCondition( int where )
{
	subConditions.RemoveAt( where );
	subCondType.RemoveAt( where );
	m_size--;
}

// returns subCondNumber when succeeded, else -1
int ExpCondition::replaceSubCondition(int subCondNumber, int column, int comparator,
									   double referenceValue, int referenceType)
{
	// e.g., size = 5, i.e. max. index = 4
	if (subCondNumber < returnSize( ) )
	{
		at( subCondNumber )->changeValues(column, comparator, referenceValue, referenceType);
		return subCondNumber;
	}

	if (subCondNumber == returnSize() ) // one is added
	{
		addSubCondition( column, comparator, referenceValue );
		return subCondNumber;
	}

	return -1;
}

int ExpCondition::replaceSubCondition(int subCondNumber, int column, int comparator,
									   CString referenceValueText, int referenceType)
{
	// e.g., size = 5, i.e. max. index = 4
	if (subCondNumber < returnSize() )
	{
		at( subCondNumber )->changeValues(column, comparator, referenceValueText, referenceType);
		return subCondNumber;
	}

	if (subCondNumber == returnSize() ) // one is added
	{
		addSubCondition( column, comparator, referenceValueText );
		return subCondNumber;
	}

	return -1;
}

int ExpCondition::checkArray(WTextFile *theTextFile )
{
	m_relevantAVs.RemoveAll( );
	m_aVsSize = 0;

	int lineTo;

	if( m_lineToFileEnd )
		lineTo = theTextFile->returnNumberOfLines( ) - 1;
	else
		lineTo = m_lastLine;
	
	for ( int linenumber = m_firstLine; linenumber <= lineTo; linenumber++ )
	{
		if( linenumber < theTextFile->returnNumberOfLines( ) )
			checkLine( theTextFile, linenumber );
		else
		{
			int critLine = linenumber + 1;

			return critLine;
		}
	}

	return -1;
}

int ExpCondition::WriteConditionToOutFile(OutFile *outFile, int lineNumber,
											 int columnNumber)
{
	int numberOfNeededLines = 1;
	
	// if lineNumber == 1: name of the condition in lineNumber 0
	if (lineNumber == 1)
	{
		outFile->getWord( m_name.GetBuffer( ), 0, columnNumber );
	}

	char buffer[256];
	int errorCode;

	if( m_centTendency == 6 ) // raw values
	{
		CArray<double> rawValues;
		computeRawValues( &errorCode, rawValues );

		if( errorCode < 1 )
		{
			numberOfNeededLines = rawValues.GetSize( );
			
			for( int i = 0; i < numberOfNeededLines; i++ )
			{
				sprintf( buffer, "%.6g", rawValues.GetAt( i ) );

				changeToRegionalDecSeparator( buffer );
				outFile->getWord( buffer, lineNumber + i, columnNumber );
			}
		}
		else // error
		{
			sprintf( buffer, "" );
			outFile->getWord( buffer, lineNumber, columnNumber );
		}

		return numberOfNeededLines;
	}
	else if( m_centTendency != 2 ) // not # incidences
	{
		double result = returnResult( &errorCode );
		
		if( errorCode < 1 )
			sprintf( buffer, "%.6g", result );
		else
			sprintf( buffer, "" );
	}
	else  // # incidences
		sprintf( buffer, "%.6g", returnResult( &errorCode ) );
	
	
	changeToRegionalDecSeparator( buffer );
	outFile->getWord( buffer, lineNumber, columnNumber );

	return numberOfNeededLines;
}

void ExpCondition::changeToRegionalDecSeparator( char *buffer )
{
	// For output to a text-file ART takes into account regional settings.
	if( g_separatorComma ) // setting comma as decimal-separator
	{
		for( int i=0; i < 256; i++ )
		{
			if( buffer[i] == '.' )
				buffer[i] = ',';
		}
	}
}

int ExpCondition::returnColumn(int subCondNumber)
{
	if (! at( subCondNumber )->orType)
		return at( subCondNumber )->column;	
	return 0;
}

int ExpCondition::returnColumn(int subCondNumber, int subSubCondNumber)
{
	if (at( subCondNumber )->orType)
		return ((OrSubCondition*)(at( subCondNumber )))->
			returnColumn(subSubCondNumber);
	return 0;
}

int ExpCondition::returnComparator(int subCondNumber)
{
	if (! at( subCondNumber )->orType)
		return at( subCondNumber )->comparator;
	return 0;
}

int ExpCondition::returnComparator(int subCondNumber, int subSubCondNumber)
{
	if (at( subCondNumber )->orType)
		return ((OrSubCondition*)(at( subCondNumber )))->
			returnComparator(subSubCondNumber);
	return 0;
}

int ExpCondition::returnReferenceType( int subCondNumber )
{
	if (! at( subCondNumber )->orType)
		return at( subCondNumber )->referenceType;
	return 0;
}

int ExpCondition::returnReferenceType( int subCondNumber, int subSubCondNumber )
{
	if (at( subCondNumber )->orType)
		return ((OrSubCondition*)(at( subCondNumber )))->
			returnReferenceType(subSubCondNumber);
	return 0;
}

double ExpCondition::returnReferenceValue(int subCondNumber)
{
	if (! at( subCondNumber )->orType)
	{
		return at( subCondNumber )->referenceValue;
	}

	return 0;
}

CString ExpCondition::returnReferenceValueText(int subCondNumber)
{
	if (! at( subCondNumber )->orType)
	{
		return at( subCondNumber )->referenceValueText;
	}

	return _T("");;
}

double ExpCondition::returnReferenceValue(int subCondNumber, int subSubCondNumber)
{
	if (at( subCondNumber )->orType)
	{
		return ((OrSubCondition*)(at( subCondNumber )))->
			returnReferenceValue(subSubCondNumber);
	}

	return 0;
}

CString ExpCondition::returnReferenceValueText(int subCondNumber, int subSubCondNumber)
{
	if (at( subCondNumber )->orType)
	{
		return ((OrSubCondition*)(at( subCondNumber )))->
			returnReferenceValueText(subSubCondNumber);
	}

	return 0;
}

void ExpCondition::setName(CString name)
{
	m_name = name;
}

CString ExpCondition::returnName( )
{
	return m_name;
}

// effective number (including those of the OrSubConditions)
int ExpCondition::returnNumberOfLines(void)
{
	int totalSize = 0;
	
	for (int i=0; i < returnSize(); i++)
	{
		totalSize += at(i)->size;
	}
	
	return totalSize;
}

int ExpCondition::returnNumberOfSubConds(void)
{
	return returnSize();
}

int ExpCondition::returnSubCondSize( int subCondNumber )
{
	return at( subCondNumber )->size;
}

int ExpCondition::returnSubCondStartPos(int subCondNumber)
{
	int totalSize = 0;

	if (subCondNumber < returnSize())
	{
		for (int i=0; i < subCondNumber; i++)
		{
			totalSize += at(i)->size;
		}
	}
	else
		totalSize = -1;
	
	return totalSize;
}

int ExpCondition::returnSubCondFromPos( int pos )
{
	// e.g., 4 positions => max. position = 3
	if (pos >= returnNumberOfLines())
		return -1;
	
	int startPos;
	for ( int i=0; i < returnSize(); i++ )
	{
		startPos = returnSubCondStartPos(i);
		if (startPos == pos) // hit exactly
			return i;
		if (startPos > pos) // already too far
			return i-1;
	}

	// If it was not there until now, it must be the last
	// (O.K. due to initial check of pos)
	return returnSize()-1;
}

int ExpCondition::returnRelFromAbsPos( int absPos )
{
	int result;

	if (absPos >= returnNumberOfLines())
		return -1;

	int subCond = returnSubCondFromPos( absPos );
	int startPos = returnSubCondStartPos( subCond );

	result = absPos - startPos;
	return result;
}
void ExpCondition::setColumnAv(int columnAv)
{
	m_columnAV = columnAv;
}

void ExpCondition::setSpecialAv(BOOL specialAv)
{
	m_specialAv = specialAv;
}

BOOL ExpCondition::retSpecialAv()
{
	return m_specialAv;
}

void ExpCondition::setLineToFileEnd( BOOL lineToFileEnd )
{
	m_lineToFileEnd = lineToFileEnd;
}

BOOL ExpCondition::returnLineToFileEnd()
{
	return m_lineToFileEnd;
}

void ExpCondition::setPopEst( BOOL popEst )
{
	m_popEst = popEst;
}

BOOL ExpCondition::returnPopEst()
{
	return m_popEst;
}

ComputedAv* ExpCondition::retComputedAv()
{
	return m_computedAv;
}



int ExpCondition::retColumnAv(void)
{
	return m_columnAV;
}

SubCondition *ExpCondition::at( int index )
{
	return subConditions.GetAt( index );
}

double &ExpCondition::AVsAt( int index )
{
	return m_relevantAVs.GetAt( index );
}

int ExpCondition::returnSize( )
{
	return m_size;
}

int ExpCondition::returnAVsSize( )
{
	return m_aVsSize;
}

void ExpCondition::setFirstLine( int number )
{
	m_firstLine = number;
}

void ExpCondition::setLastLine( int number )
{
	m_lastLine = number;
}

int ExpCondition::returnFirstLine( )
{
	return m_firstLine;
}

int ExpCondition::returnLastLine( )
{
	return m_lastLine;
}

void ExpCondition::setCentTendency( int centTendency )
{
	m_centTendency = centTendency;
}

int ExpCondition::returnCentTendency( )
{
	return m_centTendency;
}

void ExpCondition::Serialize( CArchive& archive )
{
	CObject::Serialize( archive );
	subCondType.Serialize( archive );
	
	if( ! archive.IsStoring( ) ) // load
	{
		archive >> m_columnAV >> m_result >> m_size >> m_aVsSize >> m_name >> m_firstLine >> m_lastLine
			>> m_centTendency >> m_specialAv >> m_lineToFileEnd >> m_popEst;
		
		subConditions.SetSize( m_size );
		for( int i=0; i < m_size; i++ )
		{
			if( subCondType.GetAt( i ) == 0 )
				subConditions[i] = new SubCondition;
			else
				subConditions[i] = new OrSubCondition;
		}

		m_computedAv = new ComputedAv( );

		m_relevantAVs.SetSize( m_aVsSize );
	}
	else // save
	{
		archive << m_columnAV << m_result << m_size << m_aVsSize << m_name << m_firstLine << m_lastLine
			<< m_centTendency << m_specialAv << m_lineToFileEnd << m_popEst;
	}

	// load and save
	for( int i=0; i < m_size; i++ )
	{
		subConditions[i]->Serialize( archive );
	}

	m_relevantAVs.Serialize( archive );
	m_computedAv->Serialize( archive );

}

InputError ExpCondition::checkValidInputs( WTextFile * theTextFile )
{
	InputError inputError;

	// Check subconditions
	for( int i=0; i < returnSize(); i++ )
	{
		inputError = at( i )->checkValidInputs( theTextFile ); // ExpCondition::at( i ) = SubCondition
		if( inputError.errorCode )
		{
			inputError.subCondition = i;
			break;
		}
	}

	// Check columnAv or specialAv
	if( m_specialAv )
	{
		int colToCheck;

		for( int i=0; i < m_computedAv->columns.GetCount(); i++ )
		{
			colToCheck = m_computedAv->columns.GetAt( i ) -1;

			if( (colToCheck < 0) || (colToCheck > (theTextFile->returnNumberOfColumns() - 1)) )
				inputError = handleInputError( INPUT_ERROR_SPECIAL_DV );
		}
	}
	else
	{
		if( (m_columnAV < 0) || (m_columnAV > (theTextFile->returnNumberOfColumns() - 1)) )
			inputError = handleInputError( INPUT_ERROR_COLUMN_DV );
	
		// Check 'line from'
		if( (m_firstLine < 0) || (m_firstLine > (theTextFile->returnNumberOfLines( ) - 1)) )
				inputError = handleInputError( INPUT_ERROR_LINE_FROM );

		// Check 'line to'
		if( ! m_lineToFileEnd )
		{
			if( (m_lastLine < 0) || (m_lastLine > (theTextFile->returnNumberOfLines( ) - 1)) )
					inputError = handleInputError( INPUT_ERROR_LINE_TO );
		}
	}
	
	
	return inputError;
}