#pragma once
#pragma warning( disable : 4018 4267 )

#include <afxtempl.h>
#include <atlstr.h>
#include "SubCondition.h"
#include "OrSubCondition.h"
#include "WTextFile.h"
#include "OutFile.h"
#include "ComputedAv.h"
#include "InputError.h"


class ExpCondition :
	public CObject
{
public:
	DECLARE_SERIAL( ExpCondition )

	ExpCondition( ){};
	ExpCondition(CString name);
	ExpCondition( ExpCondition *copyExpCondition );
	ExpCondition( ExpCondition *copyExpCondition, int foo );
	~ExpCondition(void);

	BOOL checkLine(WTextFile *theTextFile, int lineNumber);

	double computeMean( int *errorCode );
	void computeRawValues( int *errorCode, CArray<double> &rawValues );
	int computeNumberOfIncidences( int *errorCode );
	double computeMedian( int *errorCode );
	double computeSum( int *errorCode );
	double computeVariance( BOOL populationEstimation, int *errorCode );
	double computeStandardDeviation( BOOL populationEstimation, int *errorCode );

	double returnResult( int *errorCode );
	SubCondition* addSubCondition(int column, int comparator, double referenceValue);
	SubCondition* addSubCondition(int column, int comparator, CString referenceValueText);
	SubCondition* insertSubCondition(int where, int column, int comparator, double referenceValue);
	SubCondition* insertSubCondition(int where, int column, int comparator, CString referenceValueText);
	OrSubCondition* addOrSubCondition();
	OrSubCondition* insertOrSubCondition( int where );
	void deleteLastSubCondition();
	void deleteSubCondition( int where);
	int replaceSubCondition(int subCondNumber, int column,
		int comparator, double referenceValue, int referenceType);
	int replaceSubCondition(int subCondNumber, int column,
		int comparator, CString referenceValueText, int referenceType);
	int checkArray(WTextFile *theTextFile);
	int WriteConditionToOutFile(OutFile *outFile, int lineNumber, int columnNumber);
	void changeToRegionalDecSeparator( char *buffer );
	int returnColumn(int subCondNumber);
	int returnColumn(int subCondNumber, int subSubCondNumber);
	int returnComparator(int subCondNumber);
	int returnComparator(int subCondNumber, int subSubCondNumber);
	int returnReferenceType( int subCondNumber );
	int returnReferenceType( int subCondNumber, int subSubCondNumber );
	double returnReferenceValue(int subCondNumber);
	CString returnReferenceValueText(int subCondNumber);
	double returnReferenceValue(int subCondNumber, int subSubCondNumber);
	CString returnReferenceValueText(int subCondNumber, int subSubCondNumber);
	void setName(CString name);
	CString returnName( );
	int returnNumberOfLines(void);
	int returnNumberOfSubConds(void);
	int returnSubCondSize( int subCondNumber );
	int returnSubCondStartPos(int subCondNumber);
	int returnSubCondFromPos( int pos );
	int returnRelFromAbsPos( int absPos );
	void setColumnAv(int columnAv);
	int retColumnAv(void);
	SubCondition *at( int index );
	double &AVsAt( int index );
	int returnSize( );
	int returnAVsSize( );
	void setFirstLine( int number );
	void setLastLine( int number );
	int returnFirstLine( );
	int returnLastLine( );
	void setCentTendency( int centTendency );
	int returnCentTendency( );
	void setSpecialAv(BOOL specialAv);
	BOOL retSpecialAv();
	void setLineToFileEnd( BOOL lineToFileEnd );
	BOOL returnLineToFileEnd();
	void setPopEst( BOOL popEst );
	BOOL returnPopEst();
	ComputedAv *retComputedAv();
	InputError checkValidInputs( WTextFile * theTextFile );


	virtual void Serialize( CArchive& archive );

public:
	CArray<SubCondition *> subConditions;
	CArray<int> subCondType;

	int m_columnAV;
	BOOL m_specialAv;
	BOOL m_lineToFileEnd;
	BOOL m_popEst;
	ComputedAv *m_computedAv;

	CArray<double> m_relevantAVs;
	double m_result;
	CString m_name;
	int m_size;
	int m_aVsSize;
	int m_firstLine;
	int m_lastLine;
	int m_centTendency; // 0 = mean; 1 = median; 2 = # incidences; 3 = sum; 4 = variance without popEst, 40 = variance with popEst; 5 = sd without popEst; 50 = sd with popEst; 6 = raw values
};
